Manasurge = LibStub("AceAddon-3.0"):NewAddon("Manasurge", "AceConsole-3.0", "AceEvent-3.0", "AceTimer-3.0")
local Manasurge = Manasurge
local L = LibStub("AceLocale-3.0"):GetLocale("Manasurge", false)

BINDING_HEADER_Manasurge = "Manasurge"
BINDING_NAME_TOGGLEMANASURGE = L["Enable or disable Manasurge"]

local defaults = {
        char = {
                instance = true,
                autoequip = true,
                maxwait = 10,
                sets = {
                        normal = { mh = nil, oh = nil },
                        ss = { mh = nil, oh = nil },
                }, 
                spell = nil,
                -- internal
                enabled = true,
        }
}

local SSSpellID = 27996 -- spellspurge Spell ID
local SSEnchantID = 2674 -- spellsurge item-string Enchant ID
local SSCD = 50 -- the spellsurge cooldown (hidden)

local db
local switch = { active = false, } -- set = nil, back = 0 }
local icd = 0

local KeybindHelper = {}
do
        local t = {}
        function KeybindHelper:MakeKeyBindingTable(...)
                for k in pairs(t) do t[k] = nil end
                for i = 1, select("#", ...) do
                        local key = select(i, ...)
                        if key ~= "" then
                                tinsert(t, key)
                        end
                end
                return t
        end
end

-- check for a valid itemlink of a mainhand item
local function IsValidMainhand(ilink)
        if not IsEquippableItem(ilink) then
                Manasurge:Print(ilink .. " " .. L["is not a valid itemlink for an equipable item."])
                return false
        end

        local equip_loc = select(9, GetItemInfo(ilink))

        if not (equip_loc == "INVTYPE_WEAPONMAINHAND" or equip_loc == "INVTYPE_WEAPON" or equip_loc == "INVTYPE_2HWEAPON") then
                Manasurge:Print(ilink .. " " .. L["is not a weapon."]);
                return false
        end

        return true
end

-- check for a valid itemlink of a mainhand item with a spellsurge enchant on it
local function IsValidSSMainhand(ilink)
        if (IsValidMainhand(ilink)) then
                local found, _, istring = string.find(ilink, "^|c%x+|H(.+)|h%[.+%]")
                if not found then return false end

                local enchant = select(3, strsplit(":", istring))

                if not enchant == SSEnchantID then
                        Manasurge:Print(ilink .. " " .. L["does not have a spellsurge enchant."]);
                        return false
                end

                return true
        end

        return false
end

-- check for a valid itemlink of an offhand
local function IsValidOffhand(ilink)
        if not IsEquippableItem(ilink) then
                Manasurge:Print(ilink .. " " .. L["is not a valid itemlink for an equipable item."])
                return false
        end

        local equip_loc = select(9, GetItemInfo(ilink))

        if not (equip_loc == "INVTYPE_WEAPONOFFHAND" or equip_loc == "INVTYPE_HOLDABLE" or equip_loc == "INVTYPE_SHIELD") then
                Manasurge:Print(ilink .. " " .. L["is not an offhand."]);
                return false
        end

        return true
end

-- options
Manasurge.options = {
        type="group",
        args={
                options = {
                        type="group",
                        name=L["OPTIONS_NAME"],
                        desc=L["OPTIONS_DESC"],
                        args = {
                                instance = {
                                        type="toggle",
                                        name=L["INSTANCEONLY_NAME"],
                                        desc=L["INSTANCEONLY_DESC"],
                                        get=function() return db.instance end,
                                        set=function() db.instance = not db.instance end,
                                },
                                autoequip = {
                                        type="toggle",
                                        name=L["AUTOEQUIP_NAME"],
                                        desc=L["AUTOEQUIP_DESC"],
                                        get=function() return db.autoequip end,
                                        set=function() db.autoequip = not db.autoequip end,
                                },
                                maxwait = {
                                        type="range",
                                        name=L["MAXWAIT_NAME"],
                                        desc=L["MAXWAIT_DESC"],
                                        min=0,
                                        max=20,
                                        step=1,
                                        isPercent = false,
                                        get=function() return db.maxwait end,
                                        set=function(info, v) db.maxwait = v end,
                                },
                                spell = {
                                        type="input",
                                        name=L["SPELL_NAME"],
                                        desc=L["SPELL_DESC"],
                                        usage=L["SPELL_USAGE"],
                                        get = function() return db.spell end,
                                        set = function(info, v)
                                                if select(1, GetSpellInfo(v)) then
                                                        db.spell = v
                                                end
                                        end,
                                },
                                togglekey = {
                                        type = "keybinding",
                                        name = L["TOGGLEKEY_NAME"],
                                        desc = L["TOGGLEKEY_DESC"],
                                        get = function(info) return table.concat(KeybindHelper:MakeKeyBindingTable(GetBindingKey("TOGGLEMANASURGE")), ", ") end,
                                        set = function(info, key)
                                                if key == "" then
                                                        local t = KeybindHelper:MakeKeyBindingTable(GetBindingKey("TOGGLEMANASURGE"))
                                                        for i = 1, #t do
                                                                SetBinding(t[i])
                                                        end
                                                else
                                                        SetBinding(key, "TOGGLEMANASURGE")
                                                end
                                                SaveBindings(GetCurrentBindingSet())
                                        end,
                                },
                        }
                },
                sets = {
                        type="group",
                        name=L["SWITCHED_SETS_NAME"],
                        desc=L["SWITCHED_SETS_DESC"],
                        args = {
                                normal = {
                                        type="group",
                                        name=L["NORMAL_SET_NAME"],
                                        desc=L["NORMAL_SET_DESC"],
                                        args= {
                                                mh = {
                                                        type="input",
                                                        name=L["NORMAL_MAINHAND_NAME"],
                                                        desc=L["NORMAL_MAINHAND_DESC"],
                                                        usage=L["NORMAL_MAINHAND_USAGE"],
                                                        get=function() return db.sets.normal.mh end,
                                                        set=function(info, v)
                                                                if IsValidMainhand(v) then
                                                                        db.sets.normal.mh = v
                                                                end
                                                        end,
                                                },
                                                oh = {
                                                        type="input",
                                                        name=L["NORMAL_OFFHAND_NAME"],
                                                        desc=L["NORMAL_OFFHAND_DESC"],
                                                        usage=L["NORMAL_OFFHAND_USAGE"],
                                                        get=function() return db.sets.normal.oh end,
                                                        set=function(info, v)
                                                                if IsValidOffhand(v) then
                                                                        db.sets.normal.oh = v
                                                                end
                                                        end,
                                                },
                                                clear = {
                                                        type="execute",
                                                        name=L["CLEAR_NORMAL_NAME"],
                                                        desc=L["CLEAR_NORMAL_DESC"],
                                                        func=function()
                                                                db.sets.normal.mh = nil
                                                                db.sets.normal.oh = nil
                                                        end,
                                                        disabled=function()
                                                                return not (db.sets.normal.mh or db.sets.normal.oh)
                                                        end
                                                },
                                        },
                                },
                                ss = {
                                        type="group",
                                        name=L["SPELLSURGE_SET_NAME"],
                                        desc=L["SPELLSURGE_SET_DESC"],
                                        args= {
                                                mh = {
                                                        type="input",
                                                        name=L["SPELLSURGE_MAINHAND_NAME"],
                                                        desc=L["SPELLSURGE_MAINHAND_DESC"],
                                                        usage=L["SPELLSURGE_MAINHAND_USAGE"],
                                                        get=function() return db.sets.ss.mh end,
                                                        set=function(info, v)
                                                                if IsValidSSMainhand(v) then
                                                                        db.sets.ss.mh = v
                                                                end
                                                        end,
                                                },
                                                oh = {
                                                        type="input",
                                                        name=L["SPELLSURGE_OFFHAND_NAME"],
                                                        desc=L["SPELLSURGE_OFFHAND_DESC"],
                                                        usage=L["SPELLSURGE_OFFHAND_USAGE"],
                                                        get=function() return db.sets.ss.oh end,
                                                        set=function(info, v)
                                                                if IsValidOffhand(v) then
                                                                        db.sets.ss.oh = v
                                                                end
                                                        end,
                                                },
                                                clear = {
                                                        type="execute",
                                                        name=L["CLEAR_SPELLSURGE_NAME"],
                                                        desc=L["CLEAR_SPELLSURGE_DESC"],
                                                        func=function()
                                                                db.sets.ss.mh = nil
                                                                db.sets.ss.oh = nil
                                                        end,
                                                        disabled=function()
                                                                return not (db.sets.ss.mh or db.sets.ss.oh)
                                                        end
                                                },
                                        },
                                }
                        },
                },
        },
}

local function findItem(item)
        for i = NUM_BAG_FRAMES, 0 , -1 do
                for j = GetContainerNumSlots(i), 1, -1 do
                        local link = GetContainerItemLink(i,j)
                        if link and item == link then return i, j end
                end
        end
        return false
end

local function IsItemEquipped(item)
        return (item == GetInventoryItemLink("player", GetInventorySlotInfo("MainHandSlot")) or
        item == GetInventoryItemLink("player", GetInventorySlotInfo("SecondaryHandSlot")))
end

-- equip item specified by itemlink
local function equipItem(ilink)
        if IsItemEquipped(ilink) then return end

        local bag, slot = findItem(ilink)

        if bag then 
                ClearCursor()
                PickupContainerItem(bag, slot)
                AutoEquipCursorItem()
        else
                Manasurge:Print(ilink.." "..L["cannot be found."])
        end
end

-- equip a item set (oh is optional)
local function equipSet(set)
        if set.mh then
                equipItem(set.mh)
                if set.oh then
                        equipItem(set.oh)
                end
        end
end

local function runcheck()
        return db.enabled and db.spell and (not db.instance or IsInInstance())
end

-- execute the currently scheduled switch
local function doScheduledSwitch()
        if switch.active and switch.set then
                equipSet(switch.set)
                switch.active = false
                if switch.back then
                        Manasurge:ScheduleTimer(Manasurge.ssCDDone, switch.back-GetTime())
                end
        end
end

-- schedule a switch to the specified set
local function scheduleSwitch(set, back)
        if set.mh then
                switch = { active = true, set = set, back = back }
                if db.maxwait > 0 and InCombatLockdown() then
                        Manasurge:ScheduleTimer(doScheduledSwitch, db.maxwait)
                else
                        doScheduledSwitch()
                end
        end
end

function Manasurge:ssCDDone()
        -- only switch if the normal mainhand is equipped (eg. Kael'thas fight)
        if IsItemEquipped(db.sets.normal.mh) then
                scheduleSwitch(db.sets.ss)
        end
end

function Manasurge:OnInitialize()
        LibStub("AceConfigRegistry-3.0"):RegisterOptionsTable("Manasurge", self.options)
        self:RegisterChatCommand("ms", function() LibStub("AceConfigDialog-3.0"):Open("Manasurge") end)

        self.db = LibStub("AceDB-3.0"):New("ManasurgeDB", defaults)
        db = self.db.char
end

function Manasurge:OnEnable()
        self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
        self:RegisterEvent("ACTIONBAR_UPDATE_COOLDOWN")
        self:RegisterEvent("PLAYER_REGEN_DISABLED")

        if not db.sets.normal.mh then
                self:Print(L["NORMALMH_WARNING"])
        end
        if not db.sets.ss.mh then
                self:Print(L["SPELLSURGEMH_WARNING"])
        end
        if not db.spell then
                self:Print(L["GCDSPELL_WARNING"])
        end
end

function Manasurge:OnDisable()
        self:CancelAllTimers()
        switch.active = false
        equipSet(db.sets.normal)
end

function Manasurge:ACTIONBAR_UPDATE_COOLDOWN()
        if runcheck() and switch.active then
                local dur = select(2, GetSpellCooldown(db.spell))
                if dur >= 1.4 then
                        doScheduledSwitch()
                end
        end
end

function Manasurge:COMBAT_LOG_EVENT_UNFILTERED(event, timestamp, eventtype, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, ...)
        if runcheck() and eventtype == "SPELL_PERIODIC_ENERGIZE" and icd <= GetTime() and bit.band(srcFlags, COMBATLOG_OBJECT_AFFILIATION_MINE) and select(1, ...) == SSSpellID then
                icd = GetTime() + SSCD
                scheduleSwitch(db.sets.normal, icd)
        end
end

function Manasurge:PLAYER_REGEN_DISABLED()
        if runcheck() and db.autoequip and not IsItemEquipped(db.sets.ss.mh) and (not icd or icd < GetTime()) then 
                scheduleSwitch(db.sets.ss)
        end
end

function Manasurge:Toggle()
        db.enabled = not db.enabled

        if db.enabled then
                self:Print("|cff22ff22"..L["TOGGLE_ENABLED"])
        else
                self:Print("|cffff2222"..L["TOGGLE_DISABLED"])
        end
end
