local manasurge = Manasurge

local L = AceLibrary("AceLocale-2.2"):new("manasurge")

-- check for a valid itemlink of a mainhand item
local function IsValidMainhand(ilink)
        if not IsEquippableItem(ilink) then
                manasurge:Print(ilink .. " " .. L["is not a valid itemlink for an equipable item."])
                return false
        end

        local equip_loc = select(9, GetItemInfo(ilink))

        if not (equip_loc == "INVTYPE_WEAPONMAINHAND" or equip_loc == "INVTYPE_WEAPON" or equip_loc == "INVTYPE_2HWEAPON") then
                manasurge:Print(ilink .. " " .. L["is not a weapon."]);
                return false
        end

        return true
end

-- check for a valid itemlink of a mainhand item with a spellsurge enchant on it
local function IsValidSSMainhand(ilink)
        if (IsValidMainhand(ilink)) then
                local found, _, istring = string.find(ilink, "^|c%x+|H(.+)|h%[.+%]")
                if not found then return false end

                local enchant = select(3, strsplit(":", istring))

                if not enchant == SSEnchantID then
                        manasurge:Print(ilink .. " " .. L["does not have a spellsurge enchant."]);
                        return false
                end

                return true
        end

        return false
end

-- check for a valid itemlink of an offhand
local function IsValidOffhand(ilink)
        if not IsEquippableItem(ilink) then
                manasurge:Print(ilink .. " " .. L["is not a valid itemlink for an equipable item."])
                return false
        end

        local equip_loc = select(9, GetItemInfo(ilink))

        if not (equip_loc == "INVTYPE_WEAPONOFFHAND" or equip_loc == "INVTYPE_HOLDABLE") then
                manasurge:Print(ilink .. " " .. L["is not an offhand."]);
                return false
        end

        return true
end


manasurge.options = {
        type="group",
        args={
                instance = {
                        type="toggle",
                        name=L["instance only"],
                        desc=L["work only in instances."],
                        get=function()
                                return manasurge.db.profile.instance
                        end,
                        set=function()
                                manasurge.db.profile.instance = not manasurge.db.profile.instance
                        end,
                },
                autoequip = {
                        type="toggle",
                        name=L["autoequip"],
                        desc=L["autoequip SS set when entering combat (and not on CD)."],
                        get=function()
                                return manasurge.db.profile.autoequip
                        end,
                        set=function()
                                manasurge.db.profile.autoequip = not manasurge.db.profile.autoequip
                        end,
                },
                maxwait = {
                        type="range",
                        name=L["maxwait"],
                        desc=L["maximum time to wait before forcing a set switch."],
                        min=0,
                        max=20,
                        step=1,
                        isPercent = false,
                        get=function()
                                return manasurge.db.profile.maxwait
                        end,
                        set=function(v)
                                manasurge.db.profile.maxwait = v
                        end,
                },
                sets = {
                        type="group",
                        name=L["switched sets"],
                        desc=L["switched sets"],
                        args = {
                                normal = {
                                        type="group",
                                        name=L["normal set"],
                                        desc=L["normal set"],
                                        args= {
                                                mh = {
                                                        type="text",
                                                        name=L["normal mainhand"],
                                                        desc=L["normal mainhand"],
                                                        usage=L["<itemlink>"],
                                                        get=function()
                                                                return manasurge.db.profile.sets.normal.mh
                                                        end,
                                                        set=function(v)
                                                                if IsValidMainhand(v) then
                                                                        manasurge.db.profile.sets.normal.mh = v
                                                                end
                                                        end,
                                                },
                                                oh = {
                                                        type="text",
                                                        name=L["normal offhand"],
                                                        desc=L["normal offhand"],
                                                        usage=L["<itemlink>"],
                                                        get=function()
                                                                return manasurge.db.profile.sets.normal.oh
                                                        end,
                                                        set=function(v)
                                                                if IsValidOffhand(v) then
                                                                        manasurge.db.profile.sets.normal.oh = v
                                                                end
                                                        end,
                                                },
                                                clear = {
                                                        type="execute",
                                                        name=L["clear items"],
                                                        desc=L["clear items"],
                                                        func=function()
                                                                manasurge.db.profile.sets.normal.mh = nil
                                                                manasurge.db.profile.sets.normal.oh = nil
                                                        end
                                                },
                                        },
                                },
                                ss = {
                                        type="group",
                                        name=L["spellsurge set"],
                                        desc=L["spellsurge set"],
                                        args= {
                                                mh = {
                                                        type="text",
                                                        name=L["spellsurge mainhand"],
                                                        desc=L["spellsurge mainhand"],
                                                        usage=L["<itemlink>"],
                                                        get=function()
                                                                return manasurge.db.profile.sets.ss.mh
                                                        end,
                                                        set=function(v)
                                                                if IsValidSSMainhand(v) then
                                                                        manasurge.db.profile.sets.ss.mh = v
                                                                end
                                                        end,
                                                },
                                                oh = {
                                                        type="text",
                                                        name=L["spellsurge offhand"],
                                                        desc=L["spellsurge offhand"],
                                                        usage=L["<itemlink>"],
                                                        get=function()
                                                                return manasurge.db.profile.sets.ss.oh
                                                        end,
                                                        set=function(v)
                                                                if IsValidOffhand(v) then
                                                                        manasurge.db.profile.sets.ss.oh = v
                                                                end
                                                        end,
                                                },
                                                clear = {
                                                        type="execute",
                                                        name=L["clear items"],
                                                        desc=L["clear items"],
                                                        func=function()
                                                                manasurge.db.profile.sets.ss.mh = nil
                                                                manasurge.db.profile.sets.ss.oh = nil
                                                        end
                                                },
                                        },
                                }
                        },
                },
        },
}
